From 90684137340f3c60ed6a31815a1063a90079441e Mon Sep 17 00:00:00 2001
From: Shankar Rao <shankar.rao@gmail.com>
Date: Tue, 30 Nov 2021 14:30:51 +0100
Subject: [PATCH] Add function org-speed-command-add to add/modify speed
 commands
org-keys.el: Add function org-speed-command-add to add/modify speed commands

* lisp/org-keys.el (org-speed-command-add): Create a function that
adds new speed commands or modifies existing speed commands in
`org-speed-commands'. Speed commands whose shortcut key is already
present in `org-speed-commands' are replaced at the same position,
while speed commands with brand new shortcut keys are added to the
user section of `org-speed-commands'. 

---
 doc/org-manual.org |  7 ++++---
 etc/ORG-NEWS       |  9 ++++++++-
 lisp/org-keys.el   | 22 ++++++++++++++++++++++
 3 files changed, 34 insertions(+), 4 deletions(-)

diff --git a/doc/org-manual.org b/doc/org-manual.org
index 19f42fc77..f8b13973d 100644
--- a/doc/org-manual.org
+++ b/doc/org-manual.org
@@ -18941,9 +18941,10 @@ headline, before any of the stars.
 #+vindex: org-speed-commands
 #+findex: org-speed-command-help
 Org comes with a pre-defined list of Speed Keys.  To add or modify
-Speed Keys, customize the option ~org-speed-commands~.  For more
-details, see the variable's docstring.  With Speed Keys activated,
-{{{kbd(M-x org-speed-command-help)}}}, or {{{kbd(?)}}} when point is at the
+Speed Keys, one can either use the function ~org-speed-command-add~ or
+directly customize the option ~org-speed-commands~.  For more details,
+see the variable's docstring.  With Speed Keys activated, {{{kbd(M-x
+org-speed-command-help)}}}, or {{{kbd(?)}}} when point is at the
 beginning of an Org headline, shows currently active Speed Keys,
 including the user-defined ones.
 
diff --git a/etc/ORG-NEWS b/etc/ORG-NEWS
index e618feb9a..c39af5226 100644
--- a/etc/ORG-NEWS
+++ b/etc/ORG-NEWS
@@ -94,7 +94,14 @@ argument.
 
 ~org-get-tags~ now accepts Org element or buffer position as first
 argument.
-
+*** New function ~org-speed-command-add~ to add/modify Org speed commands
+
+This function takes an alist of speed commands in the same format as
+~org-speed-commands~ and adds these speed commands to
+~org-speed-commands~. If the letter for a speed command is already
+present in ~org-speed-commands~, then the new speed command overwrites
+the previous command at the same position. Otherwise, the new command
+is added to the user section of ~org-speed-commands~.
 ** Miscellaneous
 
 *** New =transparent-image-converter= property for =dvipng=
diff --git a/lisp/org-keys.el b/lisp/org-keys.el
index 103456526..f5d69701b 100644
--- a/lisp/org-keys.el
+++ b/lisp/org-keys.el
@@ -819,6 +819,28 @@ command."
   (with-current-buffer "*Help*"
     (setq truncate-lines t)))
 
+(defun org-speed-command-add (command-list)
+  "Add speed commands in COMMAND-LIST to ``org-speed-commands''.
+
+COMMAND-LIST is an alist of speed commands with the same format
+as ``org-speed-commands''. If a command in COMMAND-LIST is
+already present in ``org-speed-commands'', then the new command
+replaces the old command at the same position. Otherwise, the new
+command is added to the user section of
+``org-speed-commands''."
+  (let* ((title "User Speed Commands")
+         (has-user-section (assoc title org-speed-commands))
+         (new-commands nil))
+    (dolist (command command-list)
+      (if-let ((keyval (assoc (car command) org-speed-commands)))
+          (setf (cdr keyval) (cdr command))
+        (push command new-commands)))
+    (when new-commands
+      (setq org-speed-commands (append (unless has-user-section
+                                         (list (list title)))
+                                       (nreverse new-commands)
+                                       org-speed-commands)))))
+
 (defun org-speed-move-safe (cmd)
   "Execute CMD, but make sure that the cursor always ends up in a headline.
 If not, return to the original position and throw an error."
-- 
2.25.1

